/* $Id: clc_strsplit.c,v 1.9 2003/05/13 12:24:11 hfuru Exp $ */
/*
 * Copyright(c) 2003, Hallvard B. Furuseth <h.b.furuseth@usit.uio.no>
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <stdlib.h>
#include <string.h>
#include "clc_assert.h"
#include "clc_string.h"


char **
clc_strsplit(const char *s, const char *delim)
{
	const char *sp;
	char *p, **ret, **rp;
	size_t i;

	clc_assert_not_null(clc_strsplit, s);
	clc_assert_not_null(clc_strsplit, delim);

	/* skip to first substring */
	s += strspn(s, delim);

	/* allocate array */
	for (i = 1, sp = s; *sp != '\0'; i++) {
		sp += strcspn(sp, delim);
		sp += strspn(sp, delim);
	}
	ret = rp = malloc(i * sizeof(char *));

	if (ret != NULL) {
		while (*s != '\0') {
			/* found new substring */
			i = strcspn(s, delim);
			*rp++ = p = malloc(i + 1);
			if (p == NULL) {
				clc_strarray_free(ret);
				return NULL;
			}
			memcpy(p, s, i);
			p[i] = '\0';
			s += i;
			s += strspn(s, delim);
		}
		*rp = NULL;
	}

	return ret;
}

void
clc_strarray_free(char **strings)
{
	char **p;
	if (strings) {
		for (p = strings; *p != NULL; ++p)
			free(*p);
		free(strings);
	}
}
