/* $Id: test.c,v 1.3 2003/06/08 11:55:40 augestad Exp $ */
/*
 * Copyright(c) 2003, Hallvard B. Furuseth <h.b.furuseth@usit.uio.no>
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "clc_string.h"


#define TEST(cond) ((cond) ? (void)0 : sorry(__LINE__, #cond))

static void
sorry(int line, const char *cond)
{
	fprintf(stderr, "string/test.c:%d: test failed: %s\n", line, cond);
	exit(EXIT_FAILURE);
}

int
main(void)
{
	char *r, *s, *t, buf[80], **strings;

	/* clc_strcasecmp() */
	TEST(clc_strcasecmp("a b C D", "A b C d") == 0);
	TEST(clc_strcasecmp("e f g h", "A b C d") != 0);
	TEST(clc_strcasecmp("a B C ",  "A b C d") < 0);
	TEST(clc_strcasecmp("1", "0") > 0);

	/* clc_strncasecmp() */
	TEST(clc_strncasecmp("a b C D", "A b C d E", 7) == 0);
	TEST(clc_strncasecmp("a b C D", "A b C d", 10) == 0);
	TEST(clc_strncasecmp("e f g h", "A b C d", 10) != 0);
	TEST(clc_strncasecmp("a B C ",  "A b C d", 7) < 0);
	TEST(clc_strncasecmp("1", "0", 2) > 0);

	/* clc_strlcpy() */
	TEST(clc_strlcpy(buf, "ab", 4) == 2 && !strcmp(buf, "ab"));
	TEST(clc_strlcpy(buf, "cd", 2) == 2 && !strcmp(buf, "c" ));
	TEST(clc_strlcpy(buf, "e",  0) == 1 && !strcmp(buf, "c" ));

	/* clc_strlcat() */
	TEST(clc_strlcat(buf, "f",  4) == 2 && !strcmp(buf, "cf"));
	TEST(clc_strlcat(buf, "gh", 4) == 4 && !strcmp(buf, "cfg"));
	TEST(clc_strlcat(buf, "i",  1) == 4 && !strcmp(buf, "cfg"));

	/* clc_strendcpy() */
	TEST(clc_strendcpy(buf, "j", buf+2) == buf+1 && !strcmp(buf, "j"));
	TEST(clc_strendcpy(buf, "kl", buf+2) == buf+2 && !strcmp(buf, "k"));
	TEST(clc_strendcpy(buf+1, "m", buf) == buf+1 && !strcmp(buf, "k"));

	/* clc_stpcpy() */
	TEST(clc_stpcpy(buf, "kl") == buf+2 && !strcmp(buf, "kl"));

	/* clc_strdup() */
	TEST((r = clc_strdup("foo")) && !strcmp(r, "foo"));
	free(r);	

	/* clc_stralloc() */
	TEST((r = clc_stralloc("x", "y", (char *)0)) && !strcmp(r, "xy"));
	free(r);	

	/* clc_strltrim() */
	strcpy(buf, " \tabc");
	TEST(clc_strltrim(buf) == buf && !strcmp(buf, "abc"));
	TEST(clc_strltrim(buf) == buf && !strcmp(buf, "abc"));

	/* clc_strrtrim() */
	strcpy(buf, "def \t");
	TEST(clc_strrtrim(buf) == buf && !strcmp(buf, "def"));

	/* clc_strtrim() */
	strcpy(buf, " def \t");
	TEST(clc_strtrim(buf) == buf && !strcmp(buf, "def"));
	TEST(clc_strtrim(buf) == buf && !strcmp(buf, "def"));

	/* clc_strnormws() */
	strcpy(buf, "\na\nb\n c \t");
	TEST(clc_strnormws(buf) == buf && !strcmp(buf, "a b c"));
	TEST(clc_strnormws(buf) == buf && !strcmp(buf, "a b c"));

	/* clc_strleft() */
	TEST(!strcmp(clc_strleft(buf, "def", 2), "de"));
	TEST(!strcmp(clc_strleft(buf, "def", 5), "def"));

	/* clc_strright() */
	TEST(!strcmp(clc_strright(buf, "ghi", 2),  "hi"));
	TEST(!strcmp(clc_strright(buf, "ghi", 5), "ghi"));

	/* clc_strsub() */
	TEST(!strcmp(clc_strsub(buf, "jkl", 1, 1), "k"));
	TEST(!strcmp(clc_strsub(buf, "mno", 1, 3), "no"));
	TEST(!strcmp(clc_strsub(buf, "jkl", 5, 3), ""));

	/* clc_strreplace() */
	TEST((r = clc_strreplace("abc", "de", "f")) && !strcmp(r, "abc"));
	free(r);
	TEST(r = clc_strreplace("abbabba", "abba", "hu"));
	TEST(!strcmp(r, "hubba"));
	free(r);
	TEST(r = clc_strreplace("hubba bubba", "ba", "xyzzy"));
	TEST(!strcmp(r, "hubxyzzy bubxyzzy"));
	free(r);
	/* Test for more than SUBS_TOP substrings */
	TEST(r=clc_strreplace("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx","x","yz"));
	TEST(strcmp("yzyzyzyzyzyzyzyzyzyzyzyzyzyzyzyzyz"
	            "yzyzyzyzyzyzyzyzyzyzyzyzyzyzyzyzyz", r) == 0);
	free(r);

	/* clc_strupper() */
	TEST(clc_strupper(strcpy(buf, "a B")) == buf && !strcmp(buf, "A B"));

	/* clc_strlower() */
	TEST(clc_strlower(strcpy(buf, "c!D")) == buf && !strcmp(buf, "c!d"));

	/* clc_strrev() */
	TEST(!strcmp(clc_strrev(strcpy(buf, "efghi")), "ihgfe"));
	TEST(!strcmp(clc_strrev(strcpy(buf, "jklm" )), "mlkj" ));
	TEST(!strcmp(clc_strrev(strcpy(buf, ""     )), ""     ));

	/* clc_memrev() */
	TEST(!strcmp(clc_memrev(strcpy(buf, "nop"), 2), "onp"));

	/* clc_strsplit() */
	strings = clc_strsplit("  ab, c ", " ,");
	TEST(strings && strings[0] && strings[1] && !strings[2]);
	TEST(!strcmp(strings[0], "ab") && !strcmp(strings[1], "c"));
	clc_strarray_free(strings);
	TEST((strings = clc_strsplit("x", "x")) && !strings[0]);
	clc_strarray_free(strings);

	/* clc_strsep() */
	s = strcpy(buf, " ef, gh ");
	TEST(!strcmp(clc_strsep(&s, " ,"), "ef"));
	TEST(!strcmp(clc_strsep(&s, " ,"), "gh"));
	TEST(clc_strsep(&s, " ,") == NULL);

	/* clc_strtok_r() */
	TEST(!strcmp(clc_strtok_r(strcpy(buf, " ij, kl "), " ,", &t), "ij"));
	TEST(!strcmp(clc_strtok_r(NULL, " ,", &t), "kl"));
	TEST(clc_strtok_r(NULL, " ,", &t) == NULL);

	/* clc_ultostr() */
	TEST(clc_ultostr(NULL, 0, 0, 2) == 1);
	TEST(clc_ultostr(buf, 2, 1, 3) == 1 && !strcmp(buf, "1"));
	TEST(clc_ultostr(buf, 2, 2, 2) == 2 && !strcmp(buf, "0"));
	TEST(clc_ultostr(buf, 0, 1, 2) == 1 && !strcmp(buf, "0"));

	/* clc_strcasestr() */
	s = "A";
    TEST(clc_strcasestr(s, "A") == s);
	s = "AAA";
    TEST(clc_strcasestr(s, "A") == s);
    TEST(clc_strcasestr(s, "a") == s);
    TEST(clc_strcasestr("AAB", "B") != NULL);
    TEST(clc_strcasestr("AAB", "b") != NULL);
    s = "AABcdEfGhIj123";
    TEST(clc_strcasestr(s, "bcdefg") == (s+2));
    TEST(clc_strcasestr(s, "") == s);
    TEST(clc_strcasestr("AA1AA2AA3", "AA2") != NULL);
    TEST(clc_strcasestr("AAA", "x") == NULL);
    TEST(clc_strcasestr("AAA", "AAAAAA") == NULL);


	return EXIT_SUCCESS;
}
